@@ -1,5 +1,6 @@
 /********************************************************************
 Copyright 2018 Roman Gilg <subdiff@gmail.com>
+Copyright 2021 David Redondo <kde@david-redondo.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,8 +22,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 namespace Globals
 {
-void setDirPath(const QString &path);
 QString dirPath();
+/**
+ * Tries to find the specified file realtive to dirPath(). Also considers presets if there is no
+ * existing file under dirPath() yet.
+ * @returns The abosolute path to a matching file if on exists or an empty string
+ */
+QString findFile(const QString &filePath);
 }
 
 #endif
