@@ -79,13 +79,20 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return replicationSourceIndex(index.row());
     case ReplicasModelRole:
         return replicasModel(output);
-    case RefreshRatesRole:
+    case RefreshRatesRole: {
         QVariantList ret;
         for (const auto rate : refreshRates(output)) {
             ret << i18n("%1 Hz", int(rate + 0.5));
         }
         return ret;
     }
+    case CapabilitiesRole:
+        return static_cast<uint32_t>(output->capabilities());
+    case OverscanRole:
+        return output->overscan();
+    case VrrPolicyRole:
+        return static_cast<uint32_t>(output->vrrPolicy());
+    }
     return QVariant();
 }
 
@@ -158,18 +165,50 @@ bool OutputModel::setData(const QModelIndex &index, const QVariant &value, int r
             return setReplicationSourceIndex(index.row(), value.toInt() - 1);
         }
         break;
-    case ScaleRole:
+    case ScaleRole: {
         bool ok;
         const qreal scale = value.toReal(&ok);
         if (ok && !qFuzzyCompare(output.ptr->scale(), scale)) {
+            const auto oldSize = output.ptr->logicalSize().toSize();
             output.ptr->setScale(scale);
             m_config->setScale(output.ptr, scale);
+            const auto newSize = output.ptr->logicalSize().toSize();
+
+            maintainSnapping(output, oldSize, newSize);
+
             Q_EMIT sizeChanged();
             Q_EMIT dataChanged(index, index, {role, SizeRole});
             return true;
         }
         break;
     }
+    case OverscanRole:
+        if (value.canConvert<uint32_t>()) {
+            Output &output = m_outputs[index.row()];
+            const uint32_t overscan = value.toUInt();
+            if (output.ptr->overscan() == overscan) {
+                return false;
+            }
+            output.ptr->setOverscan(overscan);
+            m_config->setOverscan(output.ptr, overscan);
+            Q_EMIT dataChanged(index, index, {role});
+            return true;
+        }
+        break;
+    case VrrPolicyRole:
+        if (value.canConvert<uint32_t>()) {
+            Output &output = m_outputs[index.row()];
+            const auto policy = static_cast<KScreen::Output::VrrPolicy>(value.toUInt());
+            if (output.ptr->vrrPolicy() == policy) {
+                return false;
+            }
+            output.ptr->setVrrPolicy(policy);
+            m_config->setVrrPolicy(output.ptr, policy);
+            Q_EMIT dataChanged(index, index, {role});
+            return true;
+        }
+        break;
+    }
     return false;
 }
 
@@ -193,6 +232,9 @@ QHash<int, QByteArray> OutputModel::roleNames() const
     roles[ReplicationSourceModelRole] = "replicationSourceModel";
     roles[ReplicationSourceIndexRole] = "replicationSourceIndex";
     roles[ReplicasModelRole] = "replicasModel";
+    roles[CapabilitiesRole] = "capabilities";
+    roles[OverscanRole] = "overscan";
+    roles[VrrPolicyRole] = "vrrPolicy";
     return roles;
 }
 
@@ -333,7 +375,11 @@ bool OutputModel::setResolution(int outputIndex, int resIndex)
     if (output.ptr->currentModeId() == id) {
         return false;
     }
+    const auto oldSize = output.ptr->logicalSize().toSize();
     output.ptr->setCurrentModeId(id);
+    const auto newSize = output.ptr->logicalSize().toSize();
+
+    maintainSnapping(output, oldSize, newSize);
 
     QModelIndex index = createIndex(outputIndex, 0);
     // Calling this directly ignores possible optimization when the
@@ -410,7 +456,11 @@ bool OutputModel::setRotation(int outputIndex, KScreen::Output::Rotation rotatio
     if (output.ptr->rotation() == rotation) {
         return false;
     }
+    const auto oldSize = output.ptr->logicalSize().toSize();
     output.ptr->setRotation(rotation);
+    const auto newSize = output.ptr->logicalSize().toSize();
+
+    maintainSnapping(output, oldSize, newSize);
 
     QModelIndex index = createIndex(outputIndex, 0);
     Q_EMIT dataChanged(index, index, {RotationRole, SizeRole});
@@ -952,3 +1002,33 @@ void OutputModel::snap(const Output &output, QPoint &dest)
         }
     }
 }
+
+void OutputModel::maintainSnapping(const OutputModel::Output &changedOutput, const QSize &oldSize, const QSize &newSize)
+{
+    const auto changedCenter = QRect(changedOutput.ptr->pos(), oldSize).center();
+
+    const auto dSize = newSize - oldSize;
+    const auto delta = QPoint(dSize.width(), dSize.height());
+
+    auto updated = false;
+    for (auto &output : m_outputs) {
+        if (output.ptr->id() == changedOutput.ptr->id()) {
+            continue;
+        }
+
+        const auto pos = output.ptr->pos();
+        const auto isXTranslate = pos.x() >= changedCenter.x();
+        const auto isYTranslate = pos.y() >= changedCenter.y();
+        const auto translation = QPoint(isXTranslate ? delta.x() : 0, isYTranslate ? delta.y() : 0);
+        if (translation.isNull()) {
+            continue;
+        }
+
+        output.pos = pos + translation;
+        updated = true;
+    }
+
+    if (updated) {
+        updatePositions();
+    }
+}
